@@ -31,6 +31,8 @@
         do { pth_errno_storage = errno; } while (0)
 #define errno_restore \
         do { errno = pth_errno_storage; } while (0)
+#define errno_safe(cmd) \
+        do { int __local_errno = errno; (cmd); errno = __local_errno; } while (0)
 
 /* return plus setting an errno value */
 #if defined(PTH_DEBUG)
