@@ -529,3 +529,264 @@ pid_t pth_waitpid(pid_t wpid, int *status, int options)
     return pid;
 }
 
+/* Pth variant of readv(2) */
+ssize_t pth_readv(int fd, const struct iovec *iov, int iovcnt)
+{
+    return pth_readv_ev(fd, iov, iovcnt, NULL);
+}
+
+/* Pth variant of readv(2) with extra event(s) */
+ssize_t pth_readv_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev_extra)
+{
+    struct timeval delay;
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+    fd_set fds;
+    int n;
+
+    pth_implicit_init();
+    pth_debug2("pth_readv_ev: enter from thread \"%s\"", pth_current->name);
+
+    /* first directly poll filedescriptor for readability 
+       to avoid unneccessary (and resource consuming because of context
+       switches, etc) event handling through the scheduler */
+    FD_ZERO(&fds);
+    FD_SET(fd, &fds);
+    delay.tv_sec  = 0; 
+    delay.tv_usec = 0;
+    while ((n = pth_sc(select)(fd+1, &fds, NULL, NULL, &delay)) < 0 
+           && errno == EINTR) ; 
+
+    /* when filedescriptor is still not readable, 
+       let thread sleep until it is or event occurs */
+    if (n < 1) {
+        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_READABLE|PTH_MODE_STATIC, &ev_key, fd);
+        if (ev_extra != NULL)
+            pth_event_concat(ev, ev_extra, NULL);
+        n = pth_wait(ev);
+        if (ev_extra != NULL) {
+            pth_event_isolate(ev);
+            if (!pth_event_occurred(ev))
+                return_errno(-1, EINTR);
+        }
+    }
+
+    /* now perform the actual read */
+#if PTH_FAKE_RWV
+    while ((n = pth_readv_faked(fd, iov, iovcnt)) < 0 
+           && errno == EINTR) ;
+#else
+    while ((n = pth_sc(readv)(fd, iov, iovcnt)) < 0 
+           && errno == EINTR) ;
+#endif
+
+    pth_debug2("pth_readv_ev: leave to thread \"%s\"", pth_current->name);
+    return n;
+}
+
+/* A faked version of readv(2) */
+intern size_t pth_readv_faked(int fd, const struct iovec *iov, int iovcnt)
+{
+    char *buffer;
+    size_t bytes, copy, rv;
+    int i;
+
+    /* determine total number of bytes to read */
+    bytes = 0;
+    for (i = 0; i < iovcnt; i++) {
+        if (iov[i].iov_len <= 0)
+            return_errno(-1, EINVAL);
+        bytes += iov[i].iov_len;
+    }
+    if (bytes <= 0)
+        return_errno(-1, EINVAL);
+
+    /* allocate a temporary buffer */
+    if ((buffer = malloc(bytes)) == NULL)
+        return -1;
+    
+    /* read data into temporary buffer */
+    rv = read(fd, buffer, bytes);
+
+    /* scatter read data into callers vector */
+    if (rv > 0) {
+        bytes = rv;
+        for (i = 0; i < iovcnt; i++) {
+            copy = pth_util_min(iov[i].iov_len, bytes);
+            memcpy(iov[i].iov_base, buffer, copy);
+            buffer += copy;
+            bytes  -= copy;
+            if (bytes <= 0)
+                break;
+        }
+    }
+
+    /* remove the temporary buffer */ 
+    errno_safe( free(buffer) );
+
+    /* return number of read bytes */
+    return(rv);
+}
+
+/* Pth variant of writev(2) */
+ssize_t pth_writev(int fd, const struct iovec *iov, int iovcnt)
+{
+    return pth_writev_ev(fd, iov, iovcnt, NULL);
+}
+
+/* Pth variant of writev(2) with extra event(s) */
+ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t ev_extra)
+{
+    struct timeval delay;
+    pth_event_t ev;
+    static pth_key_t ev_key = PTH_KEY_INIT;
+    fd_set fds;
+    int n;
+
+    pth_implicit_init();
+    pth_debug2("pth_writev_ev: enter from thread \"%s\"", pth_current->name);
+
+    /* first directly poll filedescriptor for writeability 
+       to avoid unneccessary (and resource consuming because of context
+       switches, etc) event handling through the scheduler */
+    FD_ZERO(&fds);
+    FD_SET(fd, &fds);
+    delay.tv_sec  = 0; 
+    delay.tv_usec = 0;
+    while ((n = pth_sc(select)(fd+1, NULL, &fds, NULL, &delay)) < 0 
+           && errno == EINTR) ; 
+
+    /* when filedescriptor is still not writeable, 
+       let thread sleep until it is or event occurs */
+    if (n < 1) {
+        ev = pth_event(PTH_EVENT_FD|PTH_UNTIL_FD_WRITEABLE|PTH_MODE_STATIC, &ev_key, fd);
+        if (ev_extra != NULL)
+            pth_event_concat(ev, ev_extra, NULL);
+        pth_wait(ev);
+        if (ev_extra != NULL) {
+            pth_event_isolate(ev);
+            if (!pth_event_occurred(ev))
+                return_errno(-1, EINTR);
+        }
+    }
+
+    /* now perform the actual write */
+#if PTH_FAKE_RWV
+    while ((n = pth_writev_faked(fd, iov, iovcnt)) < 0 
+           && errno == EINTR) ;
+#else
+    while ((n = pth_sc(writev)(fd, iov, iovcnt)) < 0 
+           && errno == EINTR) ;
+#endif
+
+    pth_debug2("pth_writev_ev: leave to thread \"%s\"", pth_current->name);
+    return n;
+}
+
+/* A faked version of writev(2) */
+intern size_t pth_writev_faked(int fd, const struct iovec *iov, int iovcnt)
+{
+    char *buffer, *cp;
+    size_t bytes, to_copy, copy, rv;
+    int i;
+
+    /* determine total number of bytes to write */
+    bytes = 0;
+    for (i = 0; i < iovcnt; i++) {
+        if (iov[i].iov_len <= 0)
+            return_errno(-1, EINVAL);
+        bytes += iov[i].iov_len;
+    }
+    if (bytes <= 0)
+        return_errno(-1, EINVAL);
+
+    /* allocate a temporary buffer to hold the data */
+    if ((buffer = malloc(bytes)) == NULL)
+        return -1;
+
+    /* concatenate the data from callers vector into buffer */
+    to_copy = bytes;
+    cp = buffer;
+    for (i = 0; i < iovcnt; i++) {
+         copy = pth_util_min(iov[i].iov_len, to_copy);
+         memcpy(cp, iov[i].iov_base, copy);
+         to_copy -= copy;
+         if (to_copy <= 0)
+             break;
+    }
+
+    /* write continuous chunck of data */
+    rv = write(fd, buffer, bytes);
+
+    /* remove the temporary buffer */ 
+    errno_safe( free(buffer) );
+
+    return(rv);
+}
+
+/* Pth variant of POSIX pread(3) */
+ssize_t pth_pread(int fd, void *buf, size_t nbytes, off_t offset)
+{
+    static pth_mutex_t mutex = PTH_MUTEX_INIT;
+    off_t old_offset;
+    ssize_t rc;
+
+    /* protect us: pth_read can yield! */
+    if (!pth_mutex_acquire(&mutex, FALSE, NULL))
+        return (-1);
+
+    /* remember current offset */
+    if ((old_offset = lseek(fd, 0, SEEK_CUR)) == (off_t)(-1)) {
+        pth_mutex_release(&mutex);
+        return (-1);
+    }
+    /* seek to requested offset */
+    if (lseek(fd, offset, SEEK_SET) == (off_t)(-1)) {
+        pth_mutex_release(&mutex);
+        return (-1);
+    }
+
+    /* perform the read operation */
+    rc = pth_read(fd, buf, nbytes);
+    
+    /* restore the old offset situation */
+    errno_safe( lseek(fd, old_offset, SEEK_SET) );
+
+    /* unprotect and return result of read */
+    pth_mutex_release(&mutex);
+    return rc;
+}
+
+/* Pth variant of POSIX pwrite(3) */
+ssize_t pth_pwrite(int fd, const void *buf, size_t nbytes, off_t offset)
+{
+    static pth_mutex_t mutex = PTH_MUTEX_INIT;
+    off_t old_offset;
+    ssize_t rc;
+
+    /* protect us: pth_write can yield! */
+    if (!pth_mutex_acquire(&mutex, FALSE, NULL))
+        return (-1);
+
+    /* remember current offset */
+    if ((old_offset = lseek(fd, 0, SEEK_CUR)) == (off_t)(-1)) {
+        pth_mutex_release(&mutex);
+        return (-1);
+    }
+    /* seek to requested offset */
+    if (lseek(fd, offset, SEEK_SET) == (off_t)(-1)) {
+        pth_mutex_release(&mutex);
+        return (-1);
+    }
+
+    /* perform the write operation */
+    rc = pth_write(fd, buf, nbytes);
+    
+    /* restore the old offset situation */
+    errno_safe( lseek(fd, old_offset, SEEK_SET) );
+
+    /* unprotect and return result of write */
+    pth_mutex_release(&mutex);
+    return rc;
+}
+
