@@ -422,7 +422,11 @@ int pth_nonblocking(int fd)
 #ifdef  O_NDELAY
 #define O_NONBLOCKING O_NDELAY
 #else
-#error "No O_NONBLOCK or O_NDELAY define available!"
+#ifdef  FNDELAY
+#define O_NONBLOCKING FNDELAY
+#else
+#error "No O_NONBLOCK, O_NDELAY or FNDELAY flag available!"
+#endif
 #endif
 #endif
     rv = TRUE;
