@@ -1045,42 +1045,15 @@ ssize_t _pthread_write(int fd, const void *buf, size_t nbytes)
     return pth_write(fd, buf, nbytes);
 }
 
-/*
-**  SPECIAL POSIX STUFF
-*/
-
-static pth_mutex_t pthread_pwrite_mutex = PTH_MUTEX_INIT;
-static pth_mutex_t pthread_pread_mutex  = PTH_MUTEX_INIT;
+ssize_t _pthread_pread(int fd, void *buf, size_t nbytes, off_t offset)
+{
+    pthread_initialize();
+    return pth_pread(fd, buf, nbytes, offset);
+}
 
 ssize_t _pthread_pwrite(int fd, const void *buf, size_t nbytes, off_t offset)
 {
-    ssize_t rc;
-
-    if (!pth_mutex_acquire(&pthread_pwrite_mutex, FALSE, NULL))
-        return (-1);
-    if (lseek(fd, offset, SEEK_SET) == -1)
-        return (-1);
-    rc = pth_write(fd, buf, nbytes);
-    errno_preserve;
-    if (!pth_mutex_release(&pthread_pwrite_mutex))
-        return (-1);
-    errno_restore;
-    return rc;
-}
-
-ssize_t _pthread_pread(int fd, void *buf, size_t nbytes, off_t offset)
-{
-    ssize_t rc;
-
-    if (!pth_mutex_acquire(&pthread_pread_mutex, FALSE, NULL))
-        return (-1);
-    if (lseek(fd, offset, SEEK_SET) == -1)
-        return (-1);
-    rc = pth_read(fd, buf, nbytes);
-    errno_preserve;
-    if (!pth_mutex_release(&pthread_pread_mutex))
-        return (-1);
-    errno_restore;
-    return rc;
+    pthread_initialize();
+    return pth_pwrite(fd, buf, nbytes, offset);
 }
 
